@@ -1,7 +1,7 @@
 #ifndef __XS_PARSE_KEYWORD_H__
 #define __XS_PARSE_KEYWORD_H__
 
-#define XSPARSEKEYWORD_ABI_VERSION 0
+#define XSPARSEKEYWORD_ABI_VERSION 1
 
 struct XSParseKeywordPieceType;
 struct XSParseKeywordPieceType {
@@ -21,26 +21,29 @@ enum {
 
 enum {
   /* skip zero */
-  XS_PARSE_KEYWORD_BLOCK = 1,     /* op */
+  XS_PARSE_KEYWORD_LITERALCHAR = 1,
+  XS_PARSE_KEYWORD_LITERALSTR,
+  XS_PARSE_KEYWORD_FAILURE = 0x0f,
+
+  XS_PARSE_KEYWORD_BLOCK = 0x10,  /* op */
   XS_PARSE_KEYWORD_ANONSUB,       /* cv */
-  /* TODO: XS_PARSE_KEYWORD_ARITHEXPR = 3 */
-  XS_PARSE_KEYWORD_TERMEXPR = 4,  /* op */
+  /* TODO: XS_PARSE_KEYWORD_ARITHEXPR = 0x12 */
+  XS_PARSE_KEYWORD_TERMEXPR = 0x13, /* op */
   XS_PARSE_KEYWORD_LISTEXPR,      /* op */
-  /* TODO: XS_PARSE_KEYWORD_FULLEXPR = 6 */
-  XS_PARSE_KEYWORD_IDENT = 7,     /* sv */
+  /* TODO: XS_PARSE_KEYWORD_FULLEXPR = 0x15 */
+  XS_PARSE_KEYWORD_IDENT = 0x16,  /* sv */
   XS_PARSE_KEYWORD_PACKAGENAME,   /* sv */
   XS_PARSE_KEYWORD_LEXVARNAME,    /* sv */
+  XS_PARSE_KEYWORD_LEXVAR,        /* padix */
   XS_PARSE_KEYWORD_ATTRS,         /* attr.name + attr.val */
   XS_PARSE_KEYWORD_VSTRING,       /* sv */
-  XS_PARSE_KEYWORD_LITERALCHAR = 0x10,
-  XS_PARSE_KEYWORD_LITERALSTR,
-  XS_PARSE_KEYWORD_OPTIONAL = 0x20,
+
+  XS_PARSE_KEYWORD_SEQUENCE = 0x80,
   XS_PARSE_KEYWORD_REPEATED,
   XS_PARSE_KEYWORD_CHOICE,
   XS_PARSE_KEYWORD_TAGGEDCHOICE,
   XS_PARSE_KEYWORD_SEPARATEDLIST,
-  XS_PARSE_KEYWORD_FAILURE = 0x2f,
-  XS_PARSE_KEYWORD_PARENSCOPE = 0x30, /* (...) */
+  XS_PARSE_KEYWORD_PARENSCOPE = 0xb0, /* (...) */
   XS_PARSE_KEYWORD_BRACKETSCOPE,      /* [...] */
   XS_PARSE_KEYWORD_BRACESCOPE,        /* {...} */
   XS_PARSE_KEYWORD_CHEVRONSCOPE,      /* <...> */
@@ -54,18 +57,23 @@ enum {
 };
 
 enum {
-  XPK_TYPEFLAG_OPT    = (1<<16),
-  XPK_TYPEFLAG_SCOPED = (1<<17),
+  XPK_TYPEFLAG_OPT      = (1<<16),
+  XPK_TYPEFLAG_SPECIAL  = (1<<17), /* on XPK_BLOCK: scoped
+                                      on XPK_LEXVAR: my */
+
   /* These three are shifted versions of perl's G_VOID, G_SCALAR, G_ARRAY */
   XPK_TYPEFLAG_G_VOID   = (1<<18),
   XPK_TYPEFLAG_G_SCALAR = (2<<18),
-  XPK_TYPEFLAG_G_ARRAY  = (3<<18), /* yes it's called ARRAY, not LIST */
+  XPK_TYPEFLAG_G_LIST   = (3<<18), /* yes it's called LIST, not ARRAY */
 };
 
-#define XPK_BLOCK_flags(flags) {.type = XS_PARSE_KEYWORD_BLOCK|(flags)}
+#define XPK_BLOCK_flags(flags) {.type = XS_PARSE_KEYWORD_BLOCK|(flags), .u.pieces = NULL}
 #define XPK_BLOCK              XPK_BLOCK_flags(0)
-#define XPK_BLOCK_SCALARCTX    XPK_BLOCK_flags(XPK_TYPEFLAG_SCOPED|XPK_TYPEFLAG_G_SCALAR)
-#define XPK_BLOCK_LISTCTX      XPK_BLOCK_flags(XPK_TYPEFLAG_SCOPED|XPK_TYPEFLAG_G_ARRAY)
+#define XPK_BLOCK_SCALARCTX    XPK_BLOCK_flags(XPK_TYPEFLAG_SPECIAL|XPK_TYPEFLAG_G_SCALAR)
+#define XPK_BLOCK_LISTCTX      XPK_BLOCK_flags(XPK_TYPEFLAG_SPECIAL|XPK_TYPEFLAG_G_LIST)
+
+#define XPK_PREFIXED_BLOCK(...) \
+  {.type = XS_PARSE_KEYWORD_BLOCK, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
 
 #define XPK_ANONSUB {.type = XS_PARSE_KEYWORD_ANONSUB}
 
@@ -74,26 +82,32 @@ enum {
 #define XPK_TERMEXPR_SCALARCTX    XPK_TERMEXPR_flags(XPK_TYPEFLAG_G_SCALAR)
 #define XPK_LISTEXPR_flags(flags) {.type = XS_PARSE_KEYWORD_LISTEXPR|(flags)}
 #define XPK_LISTEXPR              XPK_LISTEXPR_flags(0)
-#define XPK_LISTEXPR_LISTCTX      XPK_LISTEXPR_flags(XPK_TYPEFLAG_G_ARRAY)
+#define XPK_LISTEXPR_LISTCTX      XPK_LISTEXPR_flags(XPK_TYPEFLAG_G_LIST)
 
 #define XPK_IDENT    {.type = XS_PARSE_KEYWORD_IDENT}
 #define XPK_PACKAGENAME {.type = XS_PARSE_KEYWORD_PACKAGENAME}
 
 #define XPK_LEXVARNAME(kind) {.type = XS_PARSE_KEYWORD_LEXVARNAME, .u.c = kind}
 
+#define XPK_LEXVAR_MY(kind) {.type = XS_PARSE_KEYWORD_LEXVAR|XPK_TYPEFLAG_SPECIAL, .u.c = kind}
+
 #define XPK_ATTRIBUTES {.type = XS_PARSE_KEYWORD_ATTRS}
 
 #define XPK_VSTRING     {.type = XS_PARSE_KEYWORD_VSTRING}
 #define XPK_VSTRING_OPT {.type = XS_PARSE_KEYWORD_VSTRING|XPK_TYPEFLAG_OPT}
 
+#define XPK_COMMA  {.type = XS_PARSE_KEYWORD_LITERALCHAR, .u.c = ','}
 #define XPK_COLON  {.type = XS_PARSE_KEYWORD_LITERALCHAR, .u.c = ':'}
 #define XPK_EQUALS {.type = XS_PARSE_KEYWORD_LITERALCHAR, .u.c = '='}
 
-#define XPK_STRING(s) {.type = XS_PARSE_KEYWORD_LITERALSTR, .u.str = (const char *)s}
+#define XPK_LITERAL(s) {.type = XS_PARSE_KEYWORD_LITERALSTR, .u.str = (const char *)s}
+#define XPK_STRING(s)  XPK_LITERAL(s)
 
+#define XPK_SEQUENCE(...) \
+  {.type = XS_PARSE_KEYWORD_SEQUENCE, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
 /* First piece of these must be something probe-able */
 #define XPK_OPTIONAL(...) \
-  {.type = XS_PARSE_KEYWORD_OPTIONAL, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
+  {.type = XS_PARSE_KEYWORD_SEQUENCE|XPK_TYPEFLAG_OPT, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
 #define XPK_REPEATED(...) \
   {.type = XS_PARSE_KEYWORD_REPEATED, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
 /* Every piece must be probeable */
@@ -112,12 +126,23 @@ enum {
 
 #define XPK_PARENSCOPE(...) \
   {.type = XS_PARSE_KEYWORD_PARENSCOPE, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
+#define XPK_PARENSCOPE_OPT(...) \
+  {.type = XS_PARSE_KEYWORD_PARENSCOPE|XPK_TYPEFLAG_OPT, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
+
 #define XPK_BRACKETSCOPE(...) \
   {.type = XS_PARSE_KEYWORD_BRACKETSCOPE, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
+#define XPK_BRACKETSCOPE_OPT(...) \
+  {.type = XS_PARSE_KEYWORD_BRACKETSCOPE|XPK_TYPEFLAG_OPT, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
+
 #define XPK_BRACESCOPE(...) \
   {.type = XS_PARSE_KEYWORD_BRACESCOPE, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
+#define XPK_BRACESCOPE_OPT(...) \
+  {.type = XS_PARSE_KEYWORD_BRACESCOPE|XPK_TYPEFLAG_OPT, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
+
 #define XPK_CHEVRONSCOPE(...) \
   {.type = XS_PARSE_KEYWORD_CHEVRONSCOPE, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
+#define XPK_CHEVRONSCOPE_OPT(...) \
+  {.type = XS_PARSE_KEYWORD_CHEVRONSCOPE|XPK_TYPEFLAG_OPT, .u.pieces = (const struct XSParseKeywordPieceType []){ __VA_ARGS__, {0} }}
 
 typedef union {
   OP *op;
@@ -125,6 +150,7 @@ typedef union {
   SV *sv;
   int i;
   struct { SV *name; SV *value; } attr;
+  PADOFFSET padix;
 } XSParseKeywordPiece;
 
 struct XSParseKeywordHooks {
@@ -164,26 +190,20 @@ static void S_boot_xs_parse_keyword(pTHX_ double ver) {
 
   load_module(PERL_LOADMOD_NOIMPORT, newSVpvs("XS::Parse::Keyword"), versv, NULL);
 
-  svp = hv_fetchs(PL_modglobal, "XS::Parse::Keyword/ABIVERSION", 0);
+  svp = hv_fetchs(PL_modglobal, "XS::Parse::Keyword/ABIVERSION_MIN", 0);
   if(!svp)
-    croak("XS::Parse::Keyword ABI version missing");
+    croak("XS::Parse::Keyword ABI minimum version missing");
   int abi_ver = SvIV(*svp);
-  if(abi_ver != XSPARSEKEYWORD_ABI_VERSION)
-    croak("XS::Parse::Keyword ABI version mismatch - library provides %d, compiled for %d",
+
+  if(abi_ver > XSPARSEKEYWORD_ABI_VERSION)
+    croak("XS::Parse::Keyword ABI version mismatch - library supports >= %d, compiled for %d",
         abi_ver, XSPARSEKEYWORD_ABI_VERSION);
 
-  if((svp = hv_fetchs(PL_modglobal, "XS::Parse::Keyword/ABIVERSION_MIN", 0))) {
-      abi_ver = SvIV(*svp);
-      if(abi_ver > XSPARSEKEYWORD_ABI_VERSION)
-        croak("XS::Parse::Keyword ABI version mismatch - library supports >= %d, compiled for %d",
-            abi_ver, XSPARSEKEYWORD_ABI_VERSION);
-
-      svp = hv_fetchs(PL_modglobal, "XS::Parse::Keyword/ABIVERSION_MAX", 0);
-      abi_ver = SvIV(*svp);
-      if(abi_ver < 1)
-        croak("XS::Parse::Keyword ABI version mismatch - runtime requires at least version 1, library only supports %d",
-            abi_ver);
-  }
+  svp = hv_fetchs(PL_modglobal, "XS::Parse::Keyword/ABIVERSION_MAX", 0);
+  abi_ver = SvIV(*svp);
+  if(abi_ver < 1)
+    croak("XS::Parse::Keyword ABI version mismatch - runtime requires at least version 1, library only supports %d",
+        abi_ver);
 
   register_xs_parse_keyword_func = INT2PTR(void (*)(pTHX_ const char *, const struct XSParseKeywordHooks *, void *),
       SvUV(*hv_fetchs(PL_modglobal, "XS::Parse::Keyword/register()@1", 0)));
