@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2021 -- leonerd@leonerd.org.uk
 
-package XS::Parse::Keyword 0.05;
+package XS::Parse::Keyword 0.06;
 
 use v5.14;
 use warnings;
@@ -168,6 +168,7 @@ function, using a C<union> type consisting of the following fields:
          SV *name;
          SV *value;
       } attr;
+      PADOFFSET padix;
    } XSParseKeywordPiece;
 
 Which field is set depends on the type of the piece.
@@ -198,7 +199,7 @@ macros additionally take a set of typeflags; taken from the following list:
 On C<XPK_BLOCK_flags>, this will wrap the returned optree in its own lexical
 scope, by calling C<op_scope()>.
 
-=item * XPK_TYPEFLAG_G_VOID, XPK_TYPEFLAG_G_SCALAR, XPK_TYPEFLAG_G_ARRAY
+=item * XPK_TYPEFLAG_G_VOID, XPK_TYPEFLAG_G_SCALAR, XPK_TYPEFLAG_G_LIST
 
 On optree-returning types, will contextualize the returned optree to put it in
 the given context, by calling C<op_contextualize()>.
@@ -227,6 +228,25 @@ Permits the flags C<XPK_TYPEFLAG_SCOPED> and C<XPK_TYPEFLAG_G_*>.
 Shortcuts for C<XPK_BLOCK_flags()> which wrap a scalar or list-context scope
 around the block.
 
+=head2 XPK_PREFIXED_BLOCK
+
+I<structural, emits op.>
+
+   XPK_PREFIXED_BLOCK(pieces ...)
+
+Some pieces are expected, followed by a brace-delimited block of code, which
+is passed as an optree in the I<op> field. The prefix pieces are parsed first,
+and their results are passed before the block itself.
+
+The entire sequence, including the prefix items, is contained within a pair of
+C<block_start()> / C<block_end()> calls. This permits the prefix pieces to
+introduce new items into the lexical scope of the block - for example by the
+use of C<XPK_LEXVAR_MY>.
+
+A call to C<intro_my()> is automatically made at the end of the prefix pieces,
+before the block itself is parsed, ensuring any new lexical variables are now
+visible.
+
 =head2 XPK_ANONSUB
 
 I<atomic, emits op.>
@@ -334,23 +354,29 @@ character. It is passed as a L<version> SV object in the I<sv> field.
 Identical to C<XPK_VSTRING> except it is optional; if no version string is
 found then I<sv> is set to C<NULL>.
 
-=head2 XPK_COLON
+=head2 XPK_LEXVAR_MY
+
+I<atomic, emits padix.>
+
+   XPK_LEXVAR_MY(kind)
+
+A lexical variable name is expected, added to the current pad as if specified
+in a C<my> expression, and passed as the pad index in the I<padix> field.
+
+The C<kind> argument specifies what kinds of variable are permitted, as per
+C<XPK_LEXVARNAME>.
+
+=head2 XPK_COMMA, XPK_COLON, XPK_EQUALS
 
 I<atomic, can probe, emits nothing.>
 
-A literal colon character (C<:>) is expected. No argument value is passed.
+A literal character (C<,>, C<:> or C<=>) is expected. No argument value is passed.
 
-=head2 XPK_EQUALS
+=head2 XPK_LITERAL
 
 I<atomic, can probe, emits nothing.>
 
-A literal equals character (C<=>) is expected. No argument value is passed.
-
-=head2 XPK_STRING
-
-I<atomic, can probe, emits nothing.>
-
-   XPK_STRING("literal")
+   XPK_LITERAL("literal")
 
 A literal string match is expected. No argument value is passed.
 
@@ -360,6 +386,22 @@ Generally it is best reserved just for the first component of a
 C<XPK_OPTIONAL> or C<XPK_REPEATED> sequence, to provide a "secondary keyword"
 that such a repeated item can look out for.
 
+This was previously called C<XPK_STRING>, and is provided as a synonym for
+back-compatibility but new code should use this new name instead.
+
+=head2 XPK_SEQUENCE
+
+I<structural, might support probe, emits nothing.>
+
+   XPK_SEQUENCE(pieces ...)
+
+A structural type which contains a number of pieces. This is normally
+equivalent to simply placing the pieces in sequence inside their own
+container, but it is useful inside C<XPK_CHOICE> or C<XPK_TAGGEDCHOICE>.
+
+An C<XPK_SEQUENCE> supports probe if its first contained piece does; i.e.
+is transparent to probing.
+
 =head2 XPK_OPTIONAL
 
 I<structural, emits i.>
@@ -392,6 +434,10 @@ options. An ordered list of types is provided, all of which must support
 probe. This will pass an argument whose I<i> field gives the index of the
 first choice that was accepted. The first option takes the value 0.
 
+As each of the options is interpreted as an alternative, not a sequence, you
+should use C<XPK_SEQUENCE> if a sequence of multiple items should be
+considered as a single alternative.
+
 It is not an error if no choice matches. At that point, the I<i> field will be
 set to -1.
 
@@ -413,6 +459,10 @@ which is passed in the I<i> field.
 
    XPK_TAG(value)
 
+As each of the options is interpreted as an alternative, not a sequence, you
+should use C<XPK_SEQUENCE> if a sequence of multiple items should be
+considered as a single alternative.
+
 =head2 XPK_COMMALIST
 
 I<structural, emits i.>
@@ -425,7 +475,7 @@ probe (the comma itself is sufficient to indicate a repeat).
 
 =head2 XPK_PARENSCOPE
 
-I<structural, emits nothing.>
+I<structural, can probe, emits nothing.>
 
    XPK_PARENSCOPE(pieces ...)
 
@@ -434,7 +484,7 @@ parentheses as C<( ... )>. This will pass no extra arguments.
 
 =head2 XPK_BRACKETSCOPE
 
-I<structural, emits nothing.>
+I<structural, can probe, emits nothing.>
 
    XPK_BRACKETSCOPE(pieces ...)
 
@@ -443,7 +493,7 @@ square brackets as C<[ ... ]>. This will pass no extra arguments.
 
 =head2 XPK_BRACESCOPE
 
-I<structural, emits nothing.>
+I<structural, can probe, emits nothing.>
 
    XPK_BRACESCOPE(pieces ...)
 
@@ -456,7 +506,7 @@ constrained syntax that should not just accept an arbitrary block.
 
 =head2 XPK_CHEVRONSCOPE
 
-I<structural, emits nothing.>
+I<structural, can probe, emits nothing.>
 
    XPK_CHEVRONSCOPE(pieces ...)
 
@@ -467,6 +517,23 @@ Remember that expressions like C<< a > b >> are valid term expressions, so the
 contents of this scope shouldn't allow arbitrary expressions or the closing
 bracket will be ambiguous.
 
+=head2 XPK_PARENSCOPE_OPT, XPK_BRACKETSCOPE_OPT, XPK_BRACESCOPE_OPT, XPK_CHEVRONSCOPE_OPT
+
+I<structural, can probe, emits i.>
+
+   XPK_PARENSCOPE_OPT(pieces ...)
+   XPK_BRACKETSCOPE_OPT(pieces ...)
+   XPK_BRACESCOPE_OPT(pieces ...)
+   XPK_CHEVERONSCOPE_OPT(pieces ...)
+
+Each of the four C<XPK_...SCOPE> macros above has an optional variant, whose
+name is suffixed by C<_OPT>. These pass an argument whose I<i> field is either
+true or false, indicating whether the scope was found, followed by the values
+from the scope itself.
+
+This is a convenient shortcut to nesting the scope within a C<XPK_OPTIONAL>
+macro.
+
 =cut
 
 =head1 AUTHOR
